/* Geiger Counter Interface                                     BroHogan 3/28/10
 * Counts events / min. and drives LCD display with CPM and bar graph.
 * Sample period shortens with increase in counts 
 * SETUP: (Hitachi HD44780 compatable LCD)
 * +5V LCD Vdd pin 2       >>>     Gnd LCD Vss pin 1, and R/W pin 5
 * LCD RS pin 4 to D9      >>>     LCD Enable pin 6 to D8
 * LCD D4 pin 11 to D7     >>>     LCD D5 pin 12 to D6
 * LCD D6 pin 13 to D5     >>>     LCD D7 pin 14 to D4
 * 10K pot: - ends to +5V and Gnd, wiper to LCD VO pin (pin 3)
 * *INT from Gieger circuit is connected to PIN 2 and triggered as FALLING.
 */

#include "WProgram.h"                   // needed for IDE to understand a "byte"!
#include <LiquidCrystal.h>              // Sure 8x2 LCD works with this lib
#include "Custom.h"                     // tab with custom characters for bargraph

// sample periods < 10 sec cause roundong errors (3 sec minimum))
#define LONG_PERIOD      10000          // mS sample & display below 100 CPM
#define SHORT_PERIOD     5000           // mS sample & display above 100 CPM
#define FULL_SCALE       1000           // max CPM for 8 bars & overflow warning

// instanciate the library and pass pins for (RS, Enable, D4, D5, D6, D7)
//LiquidCrystal lcd(9, 8, 7, 6, 5, 4);
LiquidCrystal    lcd(5,6,9,10,11,12);

volatile boolean newEvent = false;      // true when new event was caught by ISR
unsigned long startTime, startPeriod, samplePeriod;
unsigned long currCnt, totalCPM, totalPeriods, CPM;


void setup(){
//  attachInterrupt(0,GetEvent,FALLING);  // Geiger event on pin 2 triggers interrupt
  attachInterrupt(1,GetEvent,FALLING);  // Geiger event on pin 3 triggers interrupt
  samplePeriod = LONG_PERIOD;
  lcd.begin(2,8);                       // set lib for display size (8x2)
  lcd.createChar(0, bar_0);             // load 7 custom characters in the LCD
  lcd.createChar(1, bar_1);
  lcd.createChar(2, bar_2);
  lcd.createChar(3, bar_3);
  lcd.createChar(4, bar_4);
  lcd.createChar(5, bar_5);
  lcd.createChar(6, bar_6);
  lcd.setCursor(0,0);
  lcd.clear();                          // clear the screen
  lcd.print("CPM  Total Priod");                    // display static "CPM"
  startTime = millis();                 // start timing
  startPeriod = startTime;
}


void loop(){
  if (millis() > startPeriod + samplePeriod) ProcCounts(); // period is over
  if (newEvent){                        // add to current count
    currCnt++;
    newEvent = false;
  }
  // uncomment for self check (1/104ms X 60 = 576 CPM)
  //newEvent = true;                      // simulate an interrupt
  //delay(104);                           // 104 mS = 9.61 Hz    
}


void ProcCounts(){ // aggragate totals and reset counters for each period
  CPM = (currCnt * 60) /(samplePeriod / 1000); // calc CPM for the period
  totalPeriods ++;                      // N/U - use to average over session
  totalCPM += currCnt;                  // N/U - use to average over session
  currCnt = 0;
  startPeriod = millis();               // reset time
  if (CPM > 99)samplePeriod = SHORT_PERIOD; // shorten sample period if CPM > 100  
  else samplePeriod = LONG_PERIOD;  
  displayCount();
}


void displayCount(){
  //CPM =  random(30,700);                // uncomment for TEST
  lcd.setCursor(0, 0);                  // set cursor after "CPM "
  lcd.print("CPM      #      ");                    // clear area
  lcd.setCursor(4, 0);                  // set cursor for "CPM" 
  lcd.print(CPM);                       // display CPM on line 1

  lcd.setCursor(10, 0);                  // set cursor 
  lcd.print(totalCPM);                       // display totalCPM on line 1

  lcd.setCursor(10, 1);                  // set cursor 
  lcd.print(totalPeriods);                       // display totalPeriods on line 1
 
  if (CPM <= FULL_SCALE) lcdBar(CPM);   // display bargraph on line 2
  else {                                // put warning on line 2 if > full scale
    lcd.setCursor(0,1);
    lcd.print("         ");
    lcd.setCursor(0,1);
    lcd.print(">");
    lcd.print(FULL_SCALE);
  }
}

void lcdBar(int counts){  // displays CPM as bargraph on 2nd line
  // Adapted from DeFex http://www.arduino.cc/cgi-bin/yabb2/YaBB.pl?num=1264215873/0
  int scaler = FULL_SCALE / 47;         // 8 char=47 "bars", scaler = counts/bar
  int cntPerBar = (counts / scaler);    // amount of bars needed to display the count
  int fullBlock = (cntPerBar / 6);      // divide for full "blocks" of 6 bars 
  int prtlBlock = (cntPerBar % 6 );     // calc the remainder of bars
  lcd.setCursor(0,1);                   // move cursor to first col, 2nd line
  for (int i=0; i<fullBlock; i++){
    lcd.print(5,BYTE);                  // print full blocks
  }
  lcd.print(prtlBlock,BYTE);            // print remaining bars with custom char

  for (int i=(fullBlock + 1); i<8; i++){
    lcd.print(" ");                     // blank spaces to clean up leftover
  }  
}


void GetEvent(){   // ISR triggered for each new event (count)
  newEvent = true;
}


